@@ -4,14 +4,14 @@ use warnings;
 use 5.006;
 use base 'Exporter';
 
-our $VERSION = '0.30';
+our $VERSION = '0.31';
 
 use Carp 'confess';
 use Scalar::Util 'blessed';
 use Mouse::Util qw(load_class is_class_loaded);
 
 use Mouse::Meta::Attribute;
-use Mouse::Meta::Module; # class_of()
+use Mouse::Meta::Module;
 use Mouse::Meta::Class;
 use Mouse::Object;
 use Mouse::Util::TypeConstraints;
